/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive;

import org.xmloperator.reactive.exception.BadCharInNameException;
import org.xmloperator.reactive.exception.NameAlreadyUsedException;

public interface LaunchingContext {
	
	/**
	 * Adds an argument to this LaunchingContext.
	 * An argument is a couple (String name, Object value).
	 * The name must respect the following rule, which is XML name compatible :
	 * Name ::= ([a-zA-Z] | '_') ([a-zA-Z0-9] | '_' | '-' | '.')*
	 * The value is a constant. It cannot be changed.
	 * @param name the name of the argument to add.
	 * @param value the value of the argument to add.
	 * @exception BadCharInNameException if the given name contains bad chars.
	 * @exception NameAlreadyUsedException if an argument with the given name
	 *   already exists.
	 */
	public void addArgument(String name, Object value)
			throws BadCharInNameException, NameAlreadyUsedException;

	/**
	 * Returns the value of the argument that has the given name
	 *   or null if it doesn't exist.
	 * @param argumentName the name of an agument.
	 * @return the value of the argument that has the given name
	 *   or null if it doesn't exist.
	 */
	public Object getArgumentValue(String argumentName);
}
