/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive;

/**
 * A Message is an Event that can have attachments.
 * It is broadcasted at the beginning of the instant that just follows
 *   the instant of its generation.
 */
public interface Message extends Event {

	/**
	 * Returns true if each attachment of this Message is an Attachment.
	 * The sender entityId of the Attachment is set by the ReactiveEntity
	 *   associated to the agent that generates this Message.
	 * @return true if each attachment of this Message is an Attachment.
	 */
	public boolean isSenderCertified();
	
	/**
	 * Returns true if each attachment of this Message is an ActionAttachment.
	 * This property is verified by the ReactiveEntity associated to the agent
	 *   that generates this Message.
	 * The ReactiveCommunity executes the Action during the inactive part
	 *   of the Message broadcasting instant.
	 * @return true if each attachment of this Message is an ActionAttachment.
	 */
	public boolean isActionMessage();
}
