/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.impl;

import org.xmloperator.reactive.exception.BadCharInNameException;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.DataElement;
import org.xmloperator.reactive.element.LongDataElement;

/**
 * LongDataElement implementation.
 */
public final class LongDataElementImpl extends ElementImpl implements LongDataElement {

	final long data;

	public LongDataElementImpl(String namespaceURI, String tagName,
			long data) throws BadCharInNameException {
		super(namespaceURI, tagName);
		this.data = data;
	}

	/////////////////////////////////
	// LongDataElement implementation
	/////////////////////////////////
	
	public long getData() {
		return this.data;
	}

	/////////////////////////////
	// DataElement implementation
	/////////////////////////////
	
	public String getDataType() {
		return DataElement.TYPE_LONG;
	}
	
	public String stringDataValue() {
		return Long.toString(this.data);
	}
	
	/////////////////////////
	// Element implementation
	/////////////////////////
	
	public boolean isDataElement() {
		return true;
	}

	public boolean equals(Element element) {
		if (element == this)
			return true;
		if (!(element instanceof LongDataElementImpl)
				|| !super.equals(element))
			return false;
		LongDataElementImpl otherElement = (LongDataElementImpl)element;
		return this.data == otherElement.data;
	}

	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(super.toString());
		sb.append('(');
		sb.append(this.data);
		sb.append(')');
		return sb.toString();
	}
}
