/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.spec;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.ParentElement;

import org.xmloperator.reactive.element.impl.ParentElementImpl;

/**
 * And construct of two ElementSpecification.
 */
public final class AndElementSpecification
		implements ElementSpecification {
	
	public final ElementSpecification elementSpecification1;
	public final ElementSpecification elementSpecification2;
	public final boolean isAndNot;
	
	public AndElementSpecification(
			ElementSpecification elementSpecification1,
			ElementSpecification elementSpecification2,
			boolean isAndNot) {
		this.elementSpecification1 = elementSpecification1;
		this.elementSpecification2 = elementSpecification2;
		this.isAndNot = isAndNot;
	}
	
	//////////////////////////////////////
	// ElementSpecification implementation
	//////////////////////////////////////
	
	public boolean accepts(Element element) {
		return this.elementSpecification1.accepts(element)
			&& this.isAndNot
				? !this.elementSpecification2.accepts(element)
				: this.elementSpecification2.accepts(element);
	}
	
	////////////////////////////////////
	// TranslatableObject implementation
	////////////////////////////////////

	public ParentElement translate() {
		return new ParentElementImpl(
				Element.REACTIVE_NAMESPACE,
				this.isAndNot ? "andNot" : "and",
				new Element[] {
					this.elementSpecification1.translate(),
					this.elementSpecification2.translate()});
	}

	public Class getRebuilderClass() {
		return AndElementSpecRebuilder.class;
	}
}
