/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.spec;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.ParentElement;

import org.xmloperator.reactive.element.impl.ElementImpl;
import org.xmloperator.reactive.element.impl.ParentElementImpl;

/**
 * ElementSpecification based on the parent of the Element,
 *   and, possibly, on the name.
 */
public final class ChildElementSpecification
		implements ElementSpecification {
	
	public final ElementSpecification parentElementSpecification;
	public final String namespaceURI;
	public final String localName;
	
	public ChildElementSpecification(
			ElementSpecification parentElementSpecification) {
		this.parentElementSpecification = parentElementSpecification;
		this.namespaceURI = null;
		this.localName = null;
	}
	
	public ChildElementSpecification(
			ElementSpecification parentElementSpecification,
			String namespaceURI, String localName) {
		this.parentElementSpecification = parentElementSpecification;
		this.namespaceURI =
			namespaceURI != null ? namespaceURI.intern() : null;
		this.localName = localName;
	}
	
	//////////////////////////////////////
	// ElementSpecification implementation
	//////////////////////////////////////
	
	public boolean accepts(Element element) {
		Element parent = element.getParent();
		return (parent != null)
			&& this.parentElementSpecification.accepts(parent)
			&& (this.localName == null
				|| (element.getNamespaceURI() == this.namespaceURI
					&& element.getLocalName().equals(this.localName)));
	}
	
	////////////////////////////////////
	// TranslatableObject implementation
	////////////////////////////////////

	public ParentElement translate() {
		return this.localName == null
			? new ParentElementImpl(
				Element.REACTIVE_NAMESPACE,
				"child",
				new Element[] {
					this.parentElementSpecification.translate()})
			: new ParentElementImpl(
				Element.REACTIVE_NAMESPACE,
				"childName",
				new Element[] {
					this.parentElementSpecification.translate(),
					new ElementImpl(this.namespaceURI, this.localName)});
	}

	public Class getRebuilderClass() {
		return ChildElementSpecRebuilder.class;
	}
}
