/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.spec;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.Rebuilder;
import org.xmloperator.reactive.element.TranslatableObject;

/**
 * Rebuilder of a ElementNameSpecification.
 */
public final class ElementNameSpecRebuilder implements Rebuilder {

	private String namespaceURI = null;
	private String localName = null;
	private boolean isEnded = false;

	public ElementNameSpecRebuilder() {
	}
	
	///////////////////////////
	// Rebuilder implementation
	///////////////////////////
	
	public void startElement(
			String namespaceURI,
			String localName,
			String rawName,
			Attributes atts)
			throws SAXException {
		if (this.localName != null)
			throw new SAXException(new IllegalStateException());
		this.namespaceURI = namespaceURI;
		this.localName = localName;
	}

	public TranslatableObject endElement(
			String namespaceURI,
			String localName,
			String rawName)
			throws SAXException {
		if (this.isEnded){
			if (namespaceURI != Element.REACTIVE_NAMESPACE
					|| !localName.equals("name"))
				throw new SAXException(new IllegalStateException());
			return new ElementNameSpecification(
				this.namespaceURI, this.localName);
		}
		this.isEnded = true;
		return null;
	}

	public void characters(char[] ch, int start, int length)
			throws SAXException {
		throw new SAXException(new IllegalStateException());
	}
}
