/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.spec;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.Rebuilder;
import org.xmloperator.reactive.element.TranslatableObject;

/**
 * Rebuilder of an IncludingDescendanceElementSpecification.
 */
public final class IncludingDescendanceElementSpecRebuilder
		implements Rebuilder {

	private Rebuilder elementSpecRebuilder = null;
	private ElementSpecification elementSpecification = null;

	public IncludingDescendanceElementSpecRebuilder() {
	}
	
	///////////////////////////
	// Rebuilder implementation
	///////////////////////////
	
	public void startElement(
			String namespaceURI,
			String localName,
			String rawName,
			Attributes atts)
			throws SAXException {
		if (this.elementSpecRebuilder != null)
			this.elementSpecRebuilder.startElement(
				namespaceURI, localName, rawName, atts);
		else
			try {
				if (namespaceURI != Element.REACTIVE_NAMESPACE)
					throw new IllegalStateException();
				this.elementSpecRebuilder =
					ElementSpecRebuilderFactory.newElementSpecRebuilder(
						localName);
			}
			catch (Exception exception) {
				throw new SAXException(exception);
			}
	}

	public TranslatableObject endElement(
			String namespaceURI,
			String localName,
			String rawName)
			throws SAXException {
		if (this.elementSpecification != null) {
			if (namespaceURI != Element.REACTIVE_NAMESPACE
					|| !localName.equals("includingDescendance"))
				throw new SAXException(new IllegalStateException());
			return new ChildElementSpecification(
				this.elementSpecification);
		}
		if (this.elementSpecRebuilder == null)
			throw new SAXException(new IllegalStateException());
		this.elementSpecification =
			(ElementSpecification)this.elementSpecRebuilder.endElement(
				namespaceURI, localName, rawName);
		return null;
	}

	public void characters(char[] ch, int start, int length)
			throws SAXException {
		if (this.elementSpecRebuilder != null)
			this.elementSpecRebuilder.characters(ch, start, length);
		else
			throw new SAXException(new IllegalStateException());
	}
}
