/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.util;

import java.util.Comparator;

/**
 * Comparator of names, which processes the numerical parts.
 */
public final class NameComparator implements Comparator {
	
	public static final NameComparator COMPARATOR =
		new NameComparator();

	private NameComparator() {
	}
	
	////////////////////////////
	// Comparator implementation
	////////////////////////////
	
	/**
	 * Syntax :
	 *   name ::= non_numeric (numeric non_numeric)* numeric?
	 *   numeric ::= [1-9] [0-9]*
	 *   non_numeric ::= [ char - [1-9] ]+
	 * Rules :
	 *   non_numeric : lexicographical order
	 *   numeric : numerical order
	 */
	public int compare(Object o1, Object o2) {
		String name1 = (String)o1;
		String name2 = (String)o2;
		int length1 = name1.length();
		int length2 = name2.length();
		int index = 0;
		while (true) {
			/*
			 * Non-numeric part.
			 */
			if (index == length1 || index == length2)
				return length1 == length2
					? 0
					: length1 < length2 ? -1 : +1;
			char c1 = name1.charAt(index);
			char c2 = name2.charAt(index);
			index += 1;
			boolean isNumber_c1 = (c1 >= '1' && c1 <= '9');
			boolean isNumber_c2 = (c2 >= '1' && c2 <= '9');
			if (isNumber_c1 != isNumber_c2)
				return isNumber_c2 ? -1 : +1;
			if (!isNumber_c1) {
				if (c1 == c2)
					continue;
				return c1 < c2 ? -1 : +1;
			}
			/*
			 * Numeric part.
			 */
			int n1 = c1 - '0';
			int n2 = c2 - '0';
			int index1 = index;
			while (index1 < length1) {
				c1 = name1.charAt(index1);
				if (c1 < '0' || c1 > '9')
					break;
				n1 = 10 * n1 + (c1 - '0');
				index1 += 1;
			}
			int index2 = index;
			while (index2 < length2) {
				c2 = name2.charAt(index2);
				if (c2 < '0' || c2 > '9')
					break;
				n2 = 10 * n2 + (c2 - '0');
				index2 += 1;
			}
			if (n1 != n2)
				return n1 < n2 ? -1 : +1;
			index = index1;
		}
	}
}
