/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.facet;

import org.xmloperator.reactive.exception.BadCharInNameException;
import org.xmloperator.reactive.exception.PrivilegiedAccessException;

//import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.Message;
import org.xmloperator.reactive.Terms;
import org.xmloperator.reactive.TermsAttachment;

import org.xmloperator.reactive.util.TermsAttachmentFactory;

/**
 * This Facet provides the "exit" service delivery :
 *   accept(command attach(exit [key] [target:agentId]))
 *    -> if bad key ->
 *      emit(reject (command [attach])
 *   accept(command attach(exit [key]))
 *    -> if bad key ->
 *      emit(reject (command [attach])
 */
public final class ExitFacet extends FacetImpl {

	private static final String COMMAND = "command".intern();
	private static final String REJECT = "reject".intern();

	private static final String EXIT = "exit".intern();

	static {
		BadCharInNameException.verifyName(COMMAND);
		BadCharInNameException.verifyName(REJECT);

		BadCharInNameException.verifyName(EXIT);
	}
	
	private long globalExitKey = 0;
	private long thisAgentExitKey = 0;
	private boolean isExit = false;
	
	public ExitFacet() {
	}
	
	public ExitFacet(long globalExitKey, long thisAgentExitKey) {
		this.globalExitKey = globalExitKey;
		this.thisAgentExitKey = thisAgentExitKey;
	}
	
	public void setExitKeys(long globalExitKey, long thisAgentExitKey) {
		if (this.globalExitKey != 0 || this.thisAgentExitKey != 0)
			throw new PrivilegiedAccessException();
		this.globalExitKey = globalExitKey;
		this.thisAgentExitKey = thisAgentExitKey;
	}
	
	public boolean isExit() {
		return this.isExit;
	}
	
	public void setExit() {
		this.isExit = true;
		if (this.trace != null)
			this.trace.println(this.reac.getEntityId() + " accepts to exit");
	}

	private boolean process_command_message(TermsAttachment attachment) {
		Terms terms = attachment.getTerms();
		int termCount = terms.count();
		if (termCount < 2 || termCount > 3)
			return false;
		Object service = terms.get(0);
		if (service == EXIT) {
			Object term1 = terms.get(1);
			if (!(term1 instanceof Long))
				return false;
			long key = ((Long)term1).longValue();
			if (termCount == 2) {
				if (key == this.globalExitKey)
					this.setExit();
				else
					this.reject(attachment);
			}
			else {
				Object agentId = terms.get(2);
				if (agentId == this.reac.getEntityId()) {
					if (key == this.thisAgentExitKey)
						this.setExit();
					else
						this.reject(attachment);
				}
			}
			return true;
		}
		return false;
	}
	
	private void reject(TermsAttachment attachment) {
		Message m_command = this.reac.searchMessage(true, COMMAND);
		Message m_reject = this.reac.searchMessage(true, REJECT);
		this.reac.generateMessage(m_reject,
				TermsAttachmentFactory.newTermsAttachment(
						m_command, attachment));
	}
	
	////////////////////////////
	// Facet implementation
	////////////////////////////

	public Object getService() {
		return EXIT;
	}
	
	public boolean processServiceMessage(String messageName,
			TermsAttachment attachment) {
		if (messageName == COMMAND) {
			this.process_command_message(attachment);
			return true;
		}
		return false;
	}
}
