/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.impl;

import java.io.PrintStream;

import org.xmloperator.reactive.ReactiveFactory;
import org.xmloperator.reactive.ReactiveCommunity;
import org.xmloperator.reactive.ReactiveMachine;
import org.xmloperator.reactive.MachineKeys;
import org.xmloperator.reactive.CrashingReport;

/**
 * ReactiveFactory implementation.
 */
public final class ReactiveFactoryImpl implements ReactiveFactory {
	
	/**
	 * Unique instance of ReactiveFactoryImpl.
	 */
	public static final ReactiveFactoryImpl REACTIVE_FACTORY =
		new ReactiveFactoryImpl();
	
	/**
	 * Activates the trace system.
	 * Traces are activated at a static level, i.e. for all the instances
	 *   of these classes.
	 * @param trace a PrintStream for receiving the traces.
	 * @param objectChars a char set for selecting concerned classes.
	 *   Lowcase is for standard trace. Upcase is for full trace.
	 */
	public static void setTrace(PrintStream trace, String objectChars) {
		{
			boolean _C = objectChars.indexOf('C') >= 0;
			boolean _c = objectChars.indexOf('c') >= 0;
			ReactiveCommunityImpl.setTrace((_C  | _c) ? trace : null, _C);
		}
		{
			boolean _R = objectChars.indexOf('R') >= 0;
			boolean _r = objectChars.indexOf('r') >= 0;
			ReactiveEntityImpl.setTrace((_R  | _r) ? trace : null, _R);
		}
		{
			boolean _T = objectChars.indexOf('T') >= 0;
			boolean _t = objectChars.indexOf('t') >= 0;
			AgentThread.setTrace((_T  | _t) ? trace : null, _T);
		}
		{
			boolean _M = objectChars.indexOf('M') >= 0;
			boolean _m = objectChars.indexOf('m') >= 0;
			boolean _A = objectChars.indexOf('A') >= 0;
			ReactiveMachineImpl.setTrace((_M  | _m) ? trace : null, _M, _A);
		}
	}
	
	private ReactiveFactoryImpl() {
	}
	
	/**
	 * Gives a ReactiveReporter to a ReactiveCommunity.
	 * @param community a ReactiveCommunity.
	 * @param reporter a ReactiveReporter for the given ReactiveCommunity.
	 */
	public void setReactiveCommunityReporter(
			ReactiveCommunity community, ReactiveReporter reporter) {
		((ReactiveCommunityImpl)community).setReporter(reporter);
	}
	
	/////////////////////////////////
	// ReactiveFactory implementation
	/////////////////////////////////
	
	public ReactiveMachine newReactiveMachine(int breathe_duration_ms,
			CrashingReport crashingReport, MachineKeys machineKeys) {
		return new ReactiveMachineImpl(breathe_duration_ms, crashingReport,
				machineKeys);
	}
	
	public ReactiveMachine getReactiveMachine() {
		return ReactiveMachineImpl.getCurrentlyActiveMachine();
	}
}
