/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.util;

import org.xmloperator.reactive.Message;

/**
 * A message and its attachment.
 */
public final class MessageAttachment {

	private static int hashCode(Object content) {
		if (content == null)
			return 0;
		if (content instanceof Object[]) {
			Object[] contents = (Object[])content;
			int hashCode = 0;
			for (int k = 0; k < contents.length; k++)
				hashCode += hashCode + hashCode(contents[k]);
			return hashCode;
		}
		return content.hashCode();
	}
	
	private static boolean equals(Object content1, Object content2) {
		if (content1 == content2)
			return true;
		if ((content1 instanceof Object[]) && (content2 instanceof Object[])) {
			Object[] content1s = (Object[])content1;
			Object[] content2s = (Object[])content2;
			if (content1s.length != content1s.length)
				return false;
			for (int k = 0; k < content1s.length; k++)
				if (!equals(content1s[k], content2s[k]))
					return false;
			return true;
		}
		return false;
	}

	private final Message message;
	private final Object attachment;
	
	public MessageAttachment(Message message, Object attachment) {
		this.message = message;
		this.attachment = attachment;
	}

	public Message getMessage() {
		return this.message;
	}
	
	public Object getAttachment() {
		return this.attachment;
	}
	
	////////////////////////
	// Object implementation
	////////////////////////
	
	public int hashCode() {
		int hashCode = hashCode(this.attachment);
		hashCode += hashCode + this.message.hashCode();
		return hashCode;
	}
	
	public boolean equals(Object obj) {
		if (!(obj instanceof MessageAttachment))
			return false;
		MessageAttachment other = (MessageAttachment)obj;
		return (this.message == other.message)
			&& equals(this.attachment, other.attachment);
	}
}
